.model tiny
.code
.286

; --------------------------------------------------------------
; TSR programm which displays the time in the upper right corner
; of the screen. Call clock.com again to unload.
;
; This code has to be compiled as a COM program.
;
; --------------------------------------------------------------
;                    Macros and Constants
; --------------------------------------------------------------

StartX        =    75
Int2FID       =    0F2h
VidSeg        =    0B800h


DTime         macro
              add   al, 48          ; Byte -> ASCII
              stosw                 ; Into video-ram
              endm

Dos           macro Func            ; DOS-Interrupt
              mov   ah, Func
              int   21h
              endm

GetTime       macro Reg             ; reads a register
              mov   al, Reg         ; of the RTC and puts
              out   70h, al         ; it on the stack
              in    al, 71h
              push  ax
              endm

; --------------------------------------------------------------
;                     Code-section
; --------------------------------------------------------------

              org   100h
@Start:
              jmp   @clockInit

clockact        db  0                 ; Active flag

Int08Handler  proc  far
              cmp   cs:[clockact], 0  ; active?
              jnz   @ToExit
              or    cs:[clockact], 1  ; set Active Flag 

              pusha                 ; save registers
              push  es
              push  VidSeg
              pop   es              ; set ES to video-RAM
              mov   di, 150         ; StartOffset of video-ram
              cld
              mov   ax, es:[di]     ; read color

              GetTime  4            ; read hour
              shr   al, 4
              DTime                 ; display first number

              pop   ax
              and   al, 15
              DTime                 ; display second number

              mov   al, 10          ; separator
              DTime

              GetTime  2            ; read minute
              shr   al, 4
              DTime                 ; display first number

              pop   ax
              and   al, 15
              DTime                 ; display second number

              pop   es
              popa
              and   cs:[clockact], 0
@ToExit:
              db  0EAh
              Old   dw ?,?         ; jump to old handler
Int08Handler  endp

Int2FHandler  proc  far
              cmp   ah, Int2FID    ; Compare ID
              je    @ForMe
              db  0EAh             ; if not, call old
              Old2F dw ?,?         ; handler
@ForMe:
              cmp   al, 1
              jz    @GetSegm         ; test for AL=1 
              mov   al, ah
              ror   al, 4            ; rotate AL by 4 
              iret
@GetSegm:
              push  cs               ; Segmentaddress into ES
              pop   es               ; restore es
              iret
Int2FHandler  endp

Transient     Label word

; --------------------------------------------------------------
;                     Transient Part
; --------------------------------------------------------------

@clockInit:
        mov     ah, Int2FID
        xor     al, al
        push    ax
        int     2Fh                 ; Installation check
        pop     bx
        ror     bh, 4
        or      al, al              ; test for Null 
        jz      @Install            ; do instal
        cmp     bh, al              ; own ID
        jz      @Isthere
        mov     ah, 9               ; foreign programm
        lea     dx, Msg3
        int     21h
        .exit
@Isthere:                           ; try deinstallion
        mov     al, 1               ; Segmentaddress of resi-
        int     2Fh                 ; dent program into ES
        mov     si, es              ; store in SI 

        mov     al, 8               ; Interrupt 8
        Dos     35h
        mov     bx, es
        cmp     bx, si              ; compare segments
        jnz     @wontdo

        mov     al, 2Fh             ; Interrupt 2F
        Dos     35h
        mov     bx, es
        cmp     bx, si              ; compare segments
        jnz     @wontdo

        mov     es, si              ; ES=Segment of res. prog.
        mov     al, 8
        mov     dx, es:[Old]        ; load old vector 
        mov     ds, es:[Old+02]     ;
        Dos     25h                 ; for int 8 

        mov     al, 2Fh
        mov     dx, es:[Old2F]      ; load old vector 
        mov     ds, es:[Old2F+02]
        Dos     25h                 ; for int 2F 

        Dos     49h                 ; release memory
        push    cs
        pop     ds                  ; restore DS 
        lea     dx, Msg4
        Dos     9                   ; display text 
        .exit
@wontdo:
        lea     dx, Msg2
        mov     ah, 09
        int     21h
        .exit
@Install:
        mov     al, 8
        Dos     35h                 ; read I-Vektor 8 
        mov     [Old], bx
        mov     [Old+2], es         ; store I-Vektor 8 
        lea     dx, Int08Handler
        Dos     25h                 ; set I-Vektor 8 

        mov     al, 2Fh
        Dos     35h                 ; read I-Vector 2F
        mov     [Old2F], bx
        mov     [Old2F+02], es      ; store I-Vector 2F
        lea     dx, Int2FHandler
        Dos     25h                 ; set I-Vector 2F

        mov     es, ds:[2Ch]
        Dos     49h                 ; release environment

        lea     dx, Msg
        Dos     09h                 ; Install Msg

        lea     dx, Transient
        shr     dx, 4
        inc     dx                   ; calc. size of resident
                                     ; programm

        Dos     31h                  ; terminate resident

; --------------------------------------------------------------
;                    Transient Data
; --------------------------------------------------------------

Msg     db    'Clock installed.',10,13,36
Msg2    db    'Cannot uninstall clock.',10,13,36
Msg3    db    'ID already in use.',10,13,36
Msg4    db    'Clock uninstalled.',10,13,36

end @Start
